@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 #include <gnumeric-config.h>
@@ -31,10 +31,12 @@
 #include <sheet-control-priv.h>
 #include <ranges.h>
 #include <sheet.h>
-#include <command-context-priv.h>
+#include <command-context.h>
+#include <goffice/app/go-cmd-context-impl.h>
 
 #include <gsf/gsf-impl-utils.h>
-#include <bonobo.h>
+#include <bonobo/bonobo-main.h>
+#include <bonobo/bonobo-exception.h>
 
 typedef struct {
 	WorkbookControl wb_control;
@@ -119,7 +121,7 @@ cworkbook_sheet_add (PortableServer_Servant servant, CORBA_char const *name, COR
 {
         WorkbookControl *wbcc = (WorkbookControl *)wbcc_from_servant (servant);
 	Workbook *wb = wb_control_workbook (wbcc);
-	Sheet *sheet = workbook_sheet_add (wb, NULL, TRUE);
+	Sheet *sheet = workbook_sheet_add (wb, -1, TRUE);
 
 	SHEET_FOREACH_CONTROL(sheet, sv, sc, {
 		if (wbcc == sc_wbc (sc))
@@ -152,46 +154,46 @@ cworkbook_sheets (PortableServer_Servant servant,
 }
 
 static void
-wbcc_error (GnmCmdContext *ctxt, GError *gerr)
+wbcc_error (GOCmdContext *ctxt, GError *gerr)
 {
 	WorkbookControlCORBA *wbcc = WORKBOOK_CONTROL_CORBA (ctxt);
 
-	if (gerr->domain == gnm_error_system ()) {
+	if (gerr->domain == go_error_system ()) {
 		GNOME_Gnumeric_ErrorSystem *err = GNOME_Gnumeric_ErrorSystem__alloc();
 		err->msg = CORBA_string_dup (gerr->message);
 		CORBA_exception_set (wbcc->ev, CORBA_USER_EXCEPTION,
 			ex_GNOME_Gnumeric_ErrorSystem, err);
-	} else if (gerr->domain == gnm_error_import ()) {
+	} else if (gerr->domain == go_error_import ()) {
 		GNOME_Gnumeric_ErrorRead *err = GNOME_Gnumeric_ErrorRead__alloc();
 		err->msg = CORBA_string_dup (gerr->message);
 		CORBA_exception_set (wbcc->ev, CORBA_USER_EXCEPTION,
 			ex_GNOME_Gnumeric_ErrorRead, err);
-	} else if (gerr->domain == gnm_error_export ()) {
+	} else if (gerr->domain == go_error_export ()) {
 		GNOME_Gnumeric_ErrorSave *err = GNOME_Gnumeric_ErrorSave__alloc();
 		err->msg = CORBA_string_dup (gerr->message);
 		CORBA_exception_set (wbcc->ev, CORBA_USER_EXCEPTION,
 			ex_GNOME_Gnumeric_ErrorSave, err);
+	} else if (gerr->domain == go_error_invalid ()) {
+		GNOME_Gnumeric_ErrorInvalid *err = GNOME_Gnumeric_ErrorInvalid__alloc();
+		err->msg = CORBA_string_dup (gerr->message);
+		CORBA_exception_set (wbcc->ev, CORBA_USER_EXCEPTION,
+			ex_GNOME_Gnumeric_ErrorInvalid, err);
 	} else if (gerr->domain == gnm_error_array ()) {
 		GNOME_Gnumeric_ErrorSplitsArray *err = GNOME_Gnumeric_ErrorSplitsArray__alloc();
 		err->msg = CORBA_string_dup (gerr->message);
 		CORBA_exception_set (wbcc->ev, CORBA_USER_EXCEPTION,
 			ex_GNOME_Gnumeric_ErrorSplitsArray, err);
-	} else if (gerr->domain == gnm_error_invalid ()) {
-		GNOME_Gnumeric_ErrorInvalid *err = GNOME_Gnumeric_ErrorInvalid__alloc();
-		err->msg = CORBA_string_dup (gerr->message);
-		CORBA_exception_set (wbcc->ev, CORBA_USER_EXCEPTION,
-			ex_GNOME_Gnumeric_ErrorInvalid, err);
 	}
 }
 
 static char *
-wbcc_get_password (GnmCmdContext *cc, char const* msg)
+wbcc_get_password (GOCmdContext *cc, char const* msg)
 {
 	return NULL;
 }
 
 static void
-wbcc_set_sensitive (GnmCmdContext *cc, gboolean sensitive)
+wbcc_set_sensitive (GOCmdContext *cc, gboolean sensitive)
 {
 }
 
@@ -252,7 +254,7 @@ wbcc_finalize (GObject *obj)
 }
 
 static void
-wbcc_gnm_cmd_context_init (GnmCmdContextClass *iface)
+wbcc_gnm_cmd_context_init (GOCmdContextClass *iface)
 {
 	iface->get_password	    = wbcc_get_password;
 	iface->set_sensitive	    = wbcc_set_sensitive;
@@ -313,10 +315,10 @@ wbcc_init (WorkbookControlCORBA *wbcc)
 }
 
 GSF_CLASS_FULL (WorkbookControlCORBA, workbook_control_corba,
-		wbcc_class_init, wbcc_init, 
-		WORKBOOK_CONTROL_TYPE, 0,
+		NULL, NULL, wbcc_class_init, NULL,
+		wbcc_init,  WORKBOOK_CONTROL_TYPE, 0,
 		GSF_INTERFACE (wbcc_gnm_cmd_context_init,
-			       GNM_CMD_CONTEXT_TYPE))
+			       GO_CMD_CONTEXT_TYPE))
 
 WorkbookControl *
 workbook_control_corba_new (WorkbookView *optional_view,
