@@ -1,14 +1,17 @@
-#ifndef GNUMERIC_EXCEL_BIFF_H
-#define GNUMERIC_EXCEL_BIFF_H
+/* vim: set sw=8 ts=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+#ifndef GNM_EXCEL_BIFF_H
+#define GNM_EXCEL_BIFF_H
 
 /**
  * ms-excel-biff.h: MS Excel BIFF header for Gnumeric
  * contains data about the Excel BIFF records
  *
- * Author:
+ * Authors:
+ *    Jody Goldberg (jody@gnome.org)
  *    Michael Meeks (michael@ximian.com)
  *
- * (C) 1998, 1999, 2000 Michael Meeks
+ * (C) 1998-2001 Michael Meeks
+ * (C) 2002-2005 Jody Goldberg
  **/
 
 #include "excel.h"
@@ -27,8 +30,6 @@ typedef enum { MS_BIFF_TYPE_Workbook=0, MS_BIFF_TYPE_VBModule=1, MS_BIFF_TYPE_Wo
 /* Cell / XF types */
 typedef enum { MS_BIFF_X_STYLE=0, MS_BIFF_X_CELL=1 } MsBiffXfType ;
 typedef enum { MS_BIFF_F_MS=0, MS_BIFF_F_LOTUS=1 } MsBiffFormat ;
-typedef enum { MS_BIFF_E_CONTEXT=0, MS_BIFF_E_LEFT_TO_RIGHT=1,
-	       MS_BIFF_E_RIGHT_TO_LEFT=2 } MsBiffEastern ;
 
 typedef enum {
 	MS_BIFF_F_U_NONE = 1,
@@ -54,7 +55,8 @@ typedef enum {	/* Vertical alignment */
 	MS_BIFF_V_A_TOP     = 0,
 	MS_BIFF_V_A_CENTER  = 1,
 	MS_BIFF_V_A_BOTTOM  = 2,
-	MS_BIFF_V_A_JUSTIFY = 3
+	MS_BIFF_V_A_JUSTIFY = 3,
+	MS_BIFF_V_A_DISTRIBUTED = 4
 } MsBiffVAlign;
 
 typedef enum { /* Differences to parent styles */
@@ -82,4 +84,4 @@ extern void           free_ms_biff_bof_data (MsBiffBofData *data) ;
 
 #include "biff-types.h"
 
-#endif
+#endif /* GNM_EXCEL_BIFF_H */
