@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 #include <gnumeric-config.h>
@@ -337,12 +337,13 @@ cmd_paste (WorkbookControl *wbc, GnmPasteTarget const *pt)
 				(dst.end.row - dst.start.row)+1,
 				(dst.end.col - dst.start.col)+1,
 				rows+1, cols+1);
-			gnm_cmd_context_error_invalid (GNM_CMD_CONTEXT (wbc),
+			go_cmd_context_error_invalid (GO_CMD_CONTEXT (wbc),
 				_("Unable to paste into selection"), msg);
 			g_free (msg);
 			return;
 		}
 
+		rinfo.reloc_type = GNM_EXPR_RELOCATE_STD;
 		rinfo.origin = *src_range;
 		rinfo.col_offset = dst.start.col - rinfo.origin.start.col;
 		rinfo.row_offset = dst.start.row - rinfo.origin.start.row;
@@ -375,7 +376,7 @@ cmd_paste_to_selection (WorkbookControl *wbc, SheetView *dest_sv, int paste_flag
 	GnmRange const *r;
 	GnmPasteTarget pt;
 
-	if (!(r = selection_first_range (dest_sv, GNM_CMD_CONTEXT (wbc), _("Paste"))))
+	if (!(r = selection_first_range (dest_sv, GO_CMD_CONTEXT (wbc), _("Paste"))))
 		return;
 
 	g_return_if_fail (r !=NULL);
@@ -399,8 +400,7 @@ cmd_paste_to_selection (WorkbookControl *wbc, SheetView *dest_sv, int paste_flag
  *
  * Takes the cells in the region (col,start_row):(MAX_COL,end_row)
  * and copies them @count units (possibly negative) to the right.
- */
-
+ **/
 void
 cmd_shift_rows (WorkbookControl *wbc, Sheet *sheet,
 		int col, int start_row, int end_row, int count)
@@ -408,6 +408,7 @@ cmd_shift_rows (WorkbookControl *wbc, Sheet *sheet,
 	GnmExprRelocateInfo rinfo;
 	char *desc;
 
+	rinfo.reloc_type = GNM_EXPR_RELOCATE_STD;
 	rinfo.col_offset = count;
 	rinfo.row_offset = 0;
 	rinfo.origin_sheet = rinfo.target_sheet = sheet;
@@ -446,6 +447,7 @@ cmd_shift_cols (WorkbookControl *wbc, Sheet *sheet,
 	GnmExprRelocateInfo rinfo;
 	char *desc;
 
+	rinfo.reloc_type = GNM_EXPR_RELOCATE_STD;
 	rinfo.col_offset = 0;
 	rinfo.row_offset = count;
 	rinfo.origin_sheet = rinfo.target_sheet = sheet;
