@@ -22,7 +22,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include <gnumeric.h>
 #include "dialogs.h"
 #include "help.h"
@@ -125,14 +125,16 @@ scenario_add_ok_clicked_cb (G_GNUC_UNUSED GtkWidget *button,
 		rr = value_get_rangeref (cell_range);
 
 	if (rr == NULL) {
-		gnumeric_notice (state->base.wbcg, GTK_MESSAGE_ERROR,
+		go_gtk_notice_dialog (GTK_WINDOW (state->base.dialog),
+				 GTK_MESSAGE_ERROR,
 				 _("Invalid changing cells"));
 		gnm_expr_entry_grab_focus (state->base.input_entry, TRUE);
 		return;
 	}
 
 	if (rr->a.sheet != state->base.sheet) {
-		gnumeric_notice (state->base.wbcg, GTK_MESSAGE_ERROR,
+		go_gtk_notice_dialog (GTK_WINDOW (state->base.dialog),
+				 GTK_MESSAGE_ERROR,
 				 _("Changing cells should be on the current "
 				   "sheet only."));
 		gnm_expr_entry_grab_focus (state->base.input_entry, TRUE);
@@ -143,12 +145,14 @@ scenario_add_ok_clicked_cb (G_GNUC_UNUSED GtkWidget *button,
 	name = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
 	if (scenario_name_used (state->base.sheet->scenarios, name)) {
 	        g_free (name);
-		gnumeric_notice (state->base.wbcg, GTK_MESSAGE_ERROR,
+		go_gtk_notice_dialog (GTK_WINDOW (state->base.dialog),
+				 GTK_MESSAGE_ERROR,
 				 _("Scenario name already used"));
 		goto out;
 	} else if (check_name (name)) {
 	        g_free (name);
-		gnumeric_notice (state->base.wbcg, GTK_MESSAGE_ERROR,
+		go_gtk_notice_dialog (GTK_WINDOW (state->base.dialog),
+				 GTK_MESSAGE_ERROR,
 				 _("Invalid scenario name"));
 		goto out;
 	}
@@ -173,7 +177,8 @@ scenario_add_ok_clicked_cb (G_GNUC_UNUSED GtkWidget *button,
 	cmd_scenario_add (wbc, scenario, state->base.sheet);
 
 	if (res)
-		gnumeric_notice (state->base.wbcg, GTK_MESSAGE_WARNING,
+		go_gtk_notice_dialog (GTK_WINDOW (state->base.dialog),
+				 GTK_MESSAGE_WARNING,
 				 _("Changing cells contain at least one "
 				   "expression that is not just a value. "
 				   "Note that showing the scenario will "
@@ -225,7 +230,7 @@ dialog_scenario_add (WorkbookControlGUI *wbcg)
 	if (dialog_tool_init (&state->base, wbcg, wb_control_cur_sheet (wbc),
 			      GNUMERIC_HELP_LINK_SCENARIOS_ADD,
 			      "scenario-add.glade", "ScenarioAdd",
-			      _("_Changing cells:"), NULL, error_str,
+			      error_str,
 			      "ScenarioAdd",
 			      G_CALLBACK (scenario_add_ok_clicked_cb), NULL,
 			      G_CALLBACK (scenario_add_update_sensitivity_cb),
@@ -247,7 +252,8 @@ dialog_scenario_add (WorkbookControlGUI *wbcg)
 				  strlen (buf->str));
 	g_string_free (buf, FALSE);
 
-	state->base.output_entry = NULL;
+	state->base.gdao = NULL;
+
 	gnumeric_editable_enters (GTK_WINDOW (state->base.dialog),
 				  GTK_WIDGET (state->name_entry));
 	scenario_add_update_sensitivity_cb (NULL, state);
@@ -467,7 +473,7 @@ scenarios_cancel_clicked_cb (G_GNUC_UNUSED GtkWidget *button,
 			wb_control_sheet_focus (wbc, state->base.sheet);
 
 		/* Delete a report sheet. */
-		workbook_sheet_detach (state->base.wb, sheet, FALSE);
+		workbook_sheet_delete (sheet);
 	}
 
 	/* Recover the deleted scenarios. */
@@ -561,7 +567,8 @@ scenarios_summary_clicked_cb (G_GNUC_UNUSED GtkWidget *button,
 		GNM_EXPR_ENTRY (state->base.input_entry), state->base.sheet);
 
 	if (results == NULL) {
-		gnumeric_notice (state->base.wbcg, GTK_MESSAGE_ERROR,
+		go_gtk_notice_dialog (GTK_WINDOW (state->base.dialog),
+				 GTK_MESSAGE_ERROR,
 				 _("Results entry did not contain valid "
 				   "cell names."));
 		return;
@@ -647,7 +654,7 @@ dialog_scenarios (WorkbookControlGUI *wbcg)
 	if (dialog_tool_init (&state->base, wbcg, sheet,
 			      GNUMERIC_HELP_LINK_SCENARIOS_VIEW,
 			      "scenario-manager.glade", "Scenarios",
-			      _("Result Cells:"), NULL, error_str, "Scenarios",
+			      error_str, "Scenarios",
 			      G_CALLBACK (scenarios_ok_clicked_cb),
 			      G_CALLBACK (scenarios_cancel_clicked_cb),
 			      G_CALLBACK (scenarios_update_sensitivity_cb),
@@ -689,7 +696,7 @@ dialog_scenarios (WorkbookControlGUI *wbcg)
         return;
 
  error_out:
-	gnumeric_notice (wbcg, GTK_MESSAGE_ERROR, error_str);
+	go_gtk_notice_dialog (wbcg_toplevel (wbcg), GTK_MESSAGE_ERROR, error_str);
 	g_free (state->scenario_state);
 	g_free (state);
 
