@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 #include <gnumeric-config.h>
@@ -27,7 +27,7 @@
 #include "expr-impl.h"
 #include "expr-name.h"
 #include "dependent.h"
-#include "format.h"
+#include "gnm-format.h"
 #include "func.h"
 #include "cell.h"
 #include "sheet.h"
@@ -44,6 +44,7 @@
 #include <math.h>
 #include <string.h>
 #include <stdlib.h>
+#include <goffice/utils/go-glib-extras.h>
 
 /*
  * Using pools here probably does not save anything, but it's a darn
@@ -55,9 +56,9 @@
 
 #if USE_EXPR_POOLS
 /* Memory pool for expressions.  */
-static GnmMemChunk *expression_pool;
-#define CHUNK_ALLOC(T,p) ((T*)gnm_mem_chunk_alloc (p))
-#define CHUNK_FREE(p,v) gnm_mem_chunk_free ((p), (v))
+static GOMemChunk *expression_pool;
+#define CHUNK_ALLOC(T,p) ((T*)go_mem_chunk_alloc (p))
+#define CHUNK_FREE(p,v) go_mem_chunk_free ((p), (v))
 #else
 #define CHUNK_ALLOC(T,c) g_new (T,1)
 #define CHUNK_FREE(p,v) g_free ((v))
@@ -116,7 +117,7 @@ gnm_expr_function_eq (GnmExprFunction const *a,
 #endif
 
 GnmExpr const *
-gnm_expr_new_funcall (GnmFunc *func, GnmExprList *args)
+gnm_expr_new_funcall (GnmFunc *func, GnmExprList *arg_list)
 {
 	GnmExprFunction *ans;
 	g_return_val_if_fail (func, NULL);
@@ -128,8 +129,8 @@ gnm_expr_new_funcall (GnmFunc *func, GnmExprList *args)
 	ans->ref_count = 1;
 	ans->oper = GNM_EXPR_OP_FUNCALL;
 	gnm_func_ref (func);
-	ans->func = func;;
-	ans->arg_list = args;
+	ans->func = func;
+	ans->arg_list = arg_list;
 
 	return (GnmExpr *)ans;
 }
@@ -667,8 +668,8 @@ bin_arith (GnmExpr const *expr, GnmEvalPos const *ep,
 		case GNM_EXPR_OP_EXP:
 			if (ia == 0 && ib <= 0)
 				return value_new_error_NUM (ep);
-			dres = powgnum ((gnm_float)ia, (gnm_float)ib);
-			if (!finitegnum (dres))
+			dres = gnm_pow ((gnm_float)ia, (gnm_float)ib);
+			if (!gnm_finite (dres))
 				return value_new_error_NUM (ep);
 			break;
 
@@ -706,8 +707,8 @@ bin_arith (GnmExpr const *expr, GnmEvalPos const *ep,
 			    (va < 0 && vb != (int)vb))
 				return value_new_error_NUM (ep);
 
-			res = powgnum (va, vb);
-			return finitegnum (res)
+			res = gnm_pow (va, vb);
+			return gnm_finite (res)
 				? value_new_float (res)
 				: value_new_error_NUM (ep);
 		}
@@ -904,7 +905,7 @@ static inline GnmValue *
 negate_value (GnmValue const *v)
 {
 	GnmValue *tmp;
-	GnmFormat *fmt; 
+	GOFormat *fmt; 
 
 	if (v->type == VALUE_INTEGER) {
 		int i = v->v_int.val;
@@ -925,7 +926,7 @@ negate_value (GnmValue const *v)
 
 	if (fmt != NULL) {
 		VALUE_FMT (tmp) = fmt;
-		style_format_ref (fmt);
+		go_format_ref (fmt);
 	}
 
 	return tmp;
@@ -980,8 +981,8 @@ cb_iter_percentage (GnmValue const *v, GnmEvalPos const *ep,
 
 		if (VALUE_IS_NUMBER (v)){
 			tmp = value_new_float (value_get_as_float (v) / 100);
-			VALUE_FMT (tmp) = style_format_default_percentage ();
-			style_format_ref (VALUE_FMT (tmp));
+			VALUE_FMT (tmp) = go_format_default_percentage ();
+			go_format_ref (VALUE_FMT (tmp));
 		} else
 			tmp = value_new_error_VALUE (ep);
 
@@ -1201,8 +1202,8 @@ gnm_expr_eval (GnmExpr const *expr, GnmEvalPos const *pos,
 			res = negate_value (a);
 		else {
 			res = value_new_float (value_get_as_float (a) / 100);
-			VALUE_FMT (res) = style_format_default_percentage ();
-			style_format_ref (VALUE_FMT (res));
+			VALUE_FMT (res) = go_format_default_percentage ();
+			go_format_ref (VALUE_FMT (res));
 		}
 		value_release (a);
 		return res;
@@ -1314,13 +1315,17 @@ gnm_expr_eval (GnmExpr const *expr, GnmEvalPos const *pos,
 		/* The upper left corner manages the recalc of the expr */
 		int x = expr->array.x;
 		int y = expr->array.y;
-		if (x == 0 && y == 0){
+		if (x == 0 && y == 0) {
+			GnmEvalPos range_pos = *pos;
+			range_pos.cols = expr->array.cols;
+			range_pos.rows = expr->array.rows;
+
 			/* Release old value if necessary */
 			a = expr->array.corner.value;
 			if (a != NULL)
 				value_release (a);
 
-			a = gnm_expr_eval (expr->array.corner.expr, pos,
+			a = gnm_expr_eval (expr->array.corner.expr, &range_pos,
 				flags | GNM_EXPR_EVAL_PERMIT_NON_SCALAR);
 
 			/* Store real result (cast away const)*/
@@ -1487,7 +1492,7 @@ do_expr_as_string (GString *target, GnmExpr const *expr, GnmParsePos const *pp,
 		size_t prelen = target->len;
 
 		if (v->type == VALUE_STRING) {
-			gnm_strescape (target, v->v_str.val->str);
+			go_strescape (target, v->v_str.val->str);
 			return;
 		}
 
@@ -1785,15 +1790,10 @@ cellrange_relocate (GnmValue const *v, GnmExprRelocateInfo const *rinfo)
  *
  * Either:
  *
- * GNM_EXPR_REWRITE_SHEET:
+ * GNM_EXPR_REWRITE_INVALIDATE_SHEETS:
  *
- *	Find any references to rwinfo->u.sheet and re-write them to #REF!
- *
- * or
- *
- * GNM_EXPR_REWRITE_WORKBOOK:
- *
- *	Find any references to rwinfo->u.workbook re-write them to #REF!
+ *	Find any references to sheets marked being_invalidated and
+ *      re-write them to #REF!
  *
  * or
  *
@@ -1896,13 +1896,15 @@ gnm_expr_rewrite (GnmExpr const *expr, GnmExprRewriteInfo const *rwinfo)
 		 * sitting in the undo queue, or the clipboard.  So we just
 		 * flag the name as inactive and remove the reference here.
 		 */
-		if (!nexpr->active ||
-		    (rwinfo->type == GNM_EXPR_REWRITE_SHEET && rwinfo->u.sheet == nexpr->pos.sheet) ||
-		    (rwinfo->type == GNM_EXPR_REWRITE_WORKBOOK && rwinfo->u.workbook == nexpr->pos.wb))
+		if (!nexpr->active)
 			return gnm_expr_new_constant (value_new_error_REF (NULL));
 
-		if (rwinfo->type != GNM_EXPR_REWRITE_RELOCATE)
-			return NULL;
+		if (rwinfo->rw_type == GNM_EXPR_REWRITE_INVALIDATE_SHEETS) {
+			if (nexpr->pos.sheet && nexpr->pos.sheet->being_invalidated)
+				return gnm_expr_new_constant (value_new_error_REF (NULL));
+			else
+				return NULL;
+		}
 
 		/* If the name is not officially scoped, check that it is
 		 * available in the new scope ?  */
@@ -1939,18 +1941,14 @@ gnm_expr_rewrite (GnmExpr const *expr, GnmExprRewriteInfo const *rwinfo)
 	}
 
 	case GNM_EXPR_OP_CELLREF:
-		switch (rwinfo->type) {
-		case GNM_EXPR_REWRITE_SHEET :
-			if (expr->cellref.ref.sheet == rwinfo->u.sheet)
-				return gnm_expr_new_constant (value_new_error_REF (NULL));
-			return NULL;
-
-		case GNM_EXPR_REWRITE_WORKBOOK :
-			if (expr->cellref.ref.sheet != NULL &&
-			    expr->cellref.ref.sheet->workbook == rwinfo->u.workbook)
+		switch (rwinfo->rw_type) {
+		case GNM_EXPR_REWRITE_INVALIDATE_SHEETS:
+			if (expr->cellref.ref.sheet &&
+			    expr->cellref.ref.sheet->being_invalidated)
 				return gnm_expr_new_constant (value_new_error_REF (NULL));
 			return NULL;
 
+		case GNM_EXPR_REWRITE_NAME:
 		default : {
 			GnmCellRef res = expr->cellref.ref; /* Copy */
 
@@ -1974,40 +1972,41 @@ gnm_expr_rewrite (GnmExpr const *expr, GnmExprRewriteInfo const *rwinfo)
 			GnmCellRef const *ref_a = &v->v_range.cell.a;
 			GnmCellRef const *ref_b = &v->v_range.cell.b;
 
-			if (rwinfo->type == GNM_EXPR_REWRITE_SHEET) {
-				GnmValue *v = NULL;
+			if (rwinfo->rw_type == GNM_EXPR_REWRITE_INVALIDATE_SHEETS) {
+				Sheet *sheet_a = ref_a->sheet;
+				Sheet *sheet_b = ref_b->sheet;
+				Workbook *wb;
+				gboolean hit_a = sheet_a && sheet_a->being_invalidated;
+				gboolean hit_b = sheet_b && sheet_b->being_invalidated;
+				int dir_a, dir_b;
 
-				if (ref_a->sheet == rwinfo->u.sheet) {
-					if (ref_b->sheet != NULL &&
-					    ref_b->sheet != rwinfo->u.sheet) {
-						GnmCellRef new_a = *ref_a;
-						new_a.sheet = workbook_sheet_by_index (ref_a->sheet->workbook,
-							(ref_a->sheet->index_in_wb < ref_b->sheet->index_in_wb)
-							? ref_a->sheet->index_in_wb + 1
-							: ref_a->sheet->index_in_wb - 1);
-						v = value_new_cellrange_unsafe (&new_a, ref_b);
-					}
-				} else if (ref_b->sheet == rwinfo->u.sheet) {
-					GnmCellRef new_b = *ref_b;
-					new_b.sheet = workbook_sheet_by_index (ref_b->sheet->workbook,
-						(ref_b->sheet->index_in_wb > ref_a->sheet->index_in_wb)
-						? ref_b->sheet->index_in_wb - 1
-						: ref_b->sheet->index_in_wb + 1);
-					v = value_new_cellrange_unsafe (ref_a, &new_b);
-				} else
+				if (!hit_a && !hit_b)
 					return NULL;
-				if (v == NULL)
-					v = value_new_error_REF (NULL);
-				return gnm_expr_new_constant (v);
 
-			} else if (rwinfo->type == GNM_EXPR_REWRITE_WORKBOOK) {
-				if (ref_a->sheet != NULL &&
-				    ref_a->sheet->workbook == rwinfo->u.workbook)
+				if (sheet_a == NULL || sheet_b == NULL ||
+				    sheet_a->workbook != sheet_b->workbook)
+					/* A 3D reference between workbooks?  */
 					return gnm_expr_new_constant (value_new_error_REF (NULL));
-				if (ref_b->sheet != NULL &&
-				    ref_b->sheet->workbook == rwinfo->u.workbook)
+
+				wb = sheet_a->workbook;
+				dir_a = (sheet_a->index_in_wb < sheet_b->index_in_wb) ? +1 : -1;
+				dir_b = -dir_a;
+				/* Narrow the sheet range.  */
+				while (sheet_a != sheet_b && sheet_a->being_invalidated)
+					sheet_a = workbook_sheet_by_index (wb, sheet_a->index_in_wb + dir_a);
+				while (sheet_a != sheet_b && sheet_b->being_invalidated)
+					sheet_b = workbook_sheet_by_index (wb, sheet_b->index_in_wb + dir_b);
+
+				if (sheet_a->being_invalidated)
 					return gnm_expr_new_constant (value_new_error_REF (NULL));
-				return NULL;
+				else {
+					GnmCellRef new_a = *ref_a;
+					GnmCellRef new_b = *ref_b;
+
+					new_a.sheet = sheet_a;
+					new_b.sheet = sheet_b;
+					return gnm_expr_new_constant (value_new_cellrange_unsafe (&new_a, &new_b));
+				}
 			} else
 				return cellrange_relocate (v, &rwinfo->u.relocate);
 		}
@@ -2339,7 +2338,6 @@ gnm_expr_get_range (GnmExpr const *expr)
 static GSList *
 do_gnm_expr_get_ranges (GnmExpr const *expr, GSList *ranges)
 {
-
 	switch (expr->any.oper) {
 	case GNM_EXPR_OP_RANGE_CTOR:
 	case GNM_EXPR_OP_INTERSECT:
@@ -2450,6 +2448,12 @@ gnm_expr_is_rangeref (GnmExpr const *expr)
 	};
 }
 
+GnmExprArray const *
+gnm_expr_is_array (GnmExpr const *expr)
+{
+	return (expr->any.oper == GNM_EXPR_OP_ARRAY) ? &expr->array : NULL;
+}
+
 gboolean
 gnm_expr_is_err (GnmExpr const *expr, GnmStdError err)
 {
@@ -2463,6 +2467,36 @@ gnm_expr_is_err (GnmExpr const *expr, GnmStdError err)
 	return err == err2;
 }
 
+gboolean
+gnm_expr_is_data_table (GnmExpr const *expr, GnmCellPos *c_in, GnmCellPos *r_in)
+{
+	if (expr->any.oper == GNM_EXPR_OP_FUNCALL) {
+		char const *name = gnm_func_get_name (expr->func.func);
+		if (name && 0 == strcmp (name, "table")) {
+			if (NULL != r_in) {
+				GnmExpr const *r = gnm_expr_list_nth (expr->func.arg_list, 0);
+				if (r != NULL && r->any.oper == GNM_EXPR_OP_CELLREF) {
+					r_in->col = r->cellref.ref.col;
+					r_in->row = r->cellref.ref.row;
+				} else
+					r_in->col = r_in->row = 0; /* impossible */
+			}
+			if (NULL != c_in) {
+				GnmExpr const *c = gnm_expr_list_nth (expr->func.arg_list, 1);
+				if (c != NULL && c->any.oper == GNM_EXPR_OP_CELLREF) {
+					c_in->col = c->cellref.ref.col;
+					c_in->row = c->cellref.ref.row;
+				} else
+					c_in->col = c_in->row = 0; /* impossible */
+			}
+			return TRUE;
+		}
+	}
+
+	/* Do we need anything else here ? */
+	return FALSE;
+}
+
 void
 gnm_expr_list_unref (GnmExprList *list)
 {
@@ -2763,7 +2797,7 @@ expr_init (void)
 {
 #if USE_EXPR_POOLS
 	expression_pool =
-		gnm_mem_chunk_new ("expression pool",
+		go_mem_chunk_new ("expression pool",
 				   sizeof (GnmExpr),
 				   16 * 1024 - 128);
 #endif
@@ -2791,8 +2825,8 @@ void
 expr_shutdown (void)
 {
 #if USE_EXPR_POOLS
-	gnm_mem_chunk_foreach_leak (expression_pool, cb_expression_pool_leak, NULL);
-	gnm_mem_chunk_destroy (expression_pool, FALSE);
+	go_mem_chunk_foreach_leak (expression_pool, cb_expression_pool_leak, NULL);
+	go_mem_chunk_destroy (expression_pool, FALSE);
 	expression_pool = NULL;
 #endif
 }
