@@ -27,18 +27,38 @@
 #include "commands.h"
 #include "gui-util.h"
 
-#include <libfoocanvas/foo-canvas-line.h>
-#include <libfoocanvas/foo-canvas-rect-ellipse.h>
+#include <goffice/utils/go-glib-extras.h>
+#include <goffice/cut-n-paste/foocanvas/foo-canvas-line.h>
+#include <goffice/cut-n-paste/foocanvas/foo-canvas-rect-ellipse.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkimagemenuitem.h>
 #include <gtk/gtkseparatormenuitem.h>
 #include <gtk/gtkimage.h>
+#include <gtk/gtkdnd.h>
 #include <gdk/gdkdisplay.h>
 #include <glib/gi18n.h>
 #include <math.h>
 #define GNUMERIC_ITEM "GnmPane"
 #include "item-debug.h"
 
+#define DEBUG_DND
+
+/**
+ * For now, application/x-gnumeric is disabled. It handles neither
+ * images nor graphs correctly.
+ */
+static const GtkTargetEntry drag_types_in[] =
+	{
+		{(char *) "GNUMERIC_SAME_PROC", GTK_TARGET_SAME_APP, 0},
+ 		/* {(char *) "application/x-gnumeric", 0, 0}, */
+	};
+
+static const GtkTargetEntry drag_types_out[] =
+	{
+		{(char *) "GNUMERIC_SAME_PROC", GTK_TARGET_SAME_APP, 0},
+		{(char *) "application/x-gnumeric", 0, 0},
+	};
+
 static void
 gnumeric_pane_realized (GtkWidget *widget, gpointer ignored)
 {
@@ -49,6 +69,7 @@ static void
 gnumeric_pane_header_init (GnmPane *pane, SheetControlGUI *scg,
 			   gboolean is_col_header)
 {
+	Sheet *sheet;
 	FooCanvas *canvas = gnm_simple_canvas_new (scg);
 	FooCanvasGroup *group = FOO_CANVAS_GROUP (canvas->root);
 	FooCanvasItem *item = foo_canvas_item_new (group,
@@ -69,21 +90,14 @@ gnumeric_pane_header_init (GnmPane *pane, SheetControlGUI *scg,
 		pane->row.canvas = canvas;
 		pane->row.item = ITEM_BAR (item);
 	}
-	pane->colrow_resize.points = NULL;
-	pane->colrow_resize.start  = NULL;
-	pane->colrow_resize.guide  = NULL;
+	pane->size_guide.points = NULL;
+	pane->size_guide.start  = NULL;
+	pane->size_guide.guide  = NULL;
 
-#if 0
-	/* This would be simpler, just scroll the table and the head moves too.
-	 * but it will take some cleaning up that I have no time for just now.
-	 */
-	if (is_col_header)
-		gtk_layout_set_hadjustment (GTK_LAYOUT (canvas),
-			gtk_layout_get_hadjustment (GTK_LAYOUT (pane->gcanvas)));
-	else
-		gtk_layout_set_vadjustment (GTK_LAYOUT (canvas),
-			gtk_layout_get_vadjustment (GTK_LAYOUT (pane->gcanvas)));
-#endif
+	if (NULL != scg &&
+	    NULL != (sheet = sc_sheet (SHEET_CONTROL (scg))) &&
+	    fabs (1. - sheet->last_zoom_factor_used) > 1e-6)
+		foo_canvas_set_pixels_per_unit (canvas, sheet->last_zoom_factor_used);
 
 	g_signal_connect (G_OBJECT (canvas),
 		"realize",
@@ -128,10 +142,10 @@ gnm_pane_display_obj_size_tip (GnmPane *pane, SheetObject const *so)
 	scg_object_coords_to_anchor (scg, coords, &anchor);
 	sheet_object_anchor_to_pts (&anchor, sc_sheet (SHEET_CONTROL (scg)), pts);
 	msg = g_strdup_printf (_("%.1f x %.1f pts\n%d x %d pixels"),
-		MAX (fabs (pts[3] - pts[1]) + 1., 0),
-		MAX (fabs (pts[2] - pts[0]) + 1., 0),
-		MAX ((int)floor (fabs (coords [3] - coords [1]) + 1.5), 0),
-		MAX ((int)floor (fabs (coords [2] - coords [0]) + 1.5), 0));
+		MAX (fabs (pts[2] - pts[0]), 0),
+		MAX (fabs (pts[3] - pts[1]), 0),
+		MAX ((int)floor (fabs (coords [2] - coords [0]) + 0.5), 0),
+		MAX ((int)floor (fabs (coords [3] - coords [1]) + 0.5), 0));
 	gtk_label_set_text (GTK_LABEL (pane->size_tip), msg);
 	g_free (msg);
 }
@@ -171,6 +185,123 @@ cb_ctrl_pts_free (GtkObject **ctrl_pts)
 	g_free (ctrl_pts);
 }
 
+static void
+cb_pane_drag_data_received (GtkWidget *widget, GdkDragContext *context,
+			    gint x, gint y, GtkSelectionData *selection_data,
+			    guint info, guint time, GnmPane *pane)
+{
+	double wx, wy;
+	GnmCanvas *gcanvas = pane->gcanvas;
+
+#ifdef DEBUG_DND
+	{
+		gchar *target_name = gdk_atom_name (selection_data->target);
+		g_print ("drag-data-received - %s\n", target_name);
+		g_free (target_name);
+	}
+#endif
+
+	gnm_canvas_window_to_coord (gcanvas, x, y, &wx, &wy);
+	scg_drag_data_received (gcanvas->simple.scg,
+		gtk_drag_get_source_widget (context),
+		wx, wy, selection_data);
+}
+
+static void
+cb_gnm_pane_drag_data_get (GtkWidget *widget, GdkDragContext *context,
+			   GtkSelectionData *selection_data,
+			   guint info, guint time,
+			   SheetControlGUI *scg)
+{
+#ifdef DEBUG_DND
+	gchar *target_name = gdk_atom_name (selection_data->target);
+	g_print ("drag-data-get - %s \n", target_name);
+	g_free (target_name);
+#endif
+	scg_drag_data_get (scg, selection_data);
+}
+
+/* Move the rubber bands if we are the source */
+static gboolean
+cb_gnm_pane_drag_motion (GtkWidget *widget, GdkDragContext *context,
+			 int x, int y, guint32 time, GnmPane *pane)
+{
+	GtkWidget *source_widget = gtk_drag_get_source_widget (context);
+	SheetControlGUI *scg = GNM_CANVAS (widget)->simple.scg;
+
+	if ((IS_GNM_CANVAS (source_widget) &&
+	     GNM_CANVAS (source_widget)->simple.scg == scg)) {
+		/* same scg */
+		GnmCanvas *gcanvas = GNM_CANVAS (widget);
+		GdkModifierType mask;
+		double wx, wy;
+
+		g_object_set_data (&context->parent_instance, "gnm-pane", pane);
+		gnm_canvas_window_to_coord (gcanvas, x, y, &wx, &wy);
+
+		gdk_window_get_pointer (gtk_widget_get_parent_window (widget),
+			NULL, NULL, &mask);
+		gnm_pane_objects_drag (GNM_CANVAS (source_widget)->pane, NULL,
+			wx, wy, 8, FALSE, (mask & GDK_SHIFT_MASK) != 0);
+		gdk_drag_status (context, GDK_ACTION_MOVE, time);
+	}
+	return TRUE;
+}
+
+/**
+ * Move the rubber bands back to original position when curser leaves
+ * the scg, but not when it moves to another pane. We use object data,
+ * and rely on gtk sending drag_move to the new widget before sending
+ * drag_leave to the old one.
+ */
+static void
+cb_gnm_pane_drag_leave (GtkWidget *widget, GdkDragContext *context,
+		       guint32 time, GnmPane *pane)
+{
+	GtkWidget *source_widget = gtk_drag_get_source_widget (context);
+	GnmPane *source_pane;
+	GnmPane *ctxt_pane;
+
+	if (!source_widget) return;
+
+	g_return_if_fail (IS_GNM_CANVAS (source_widget));
+
+	source_pane = GNM_CANVAS (source_widget)->pane;
+
+	ctxt_pane = g_object_get_data (&context->parent_instance, "gnm-pane");
+	if (pane != ctxt_pane)
+		return;
+	g_object_set_data (&context->parent_instance, "gnm-pane", NULL);
+
+	gnm_pane_objects_drag (source_pane, NULL,
+			       source_pane->drag.origin_x,
+			       source_pane->drag.origin_y,
+			       8, FALSE, FALSE);
+	source_pane->drag.had_motion = FALSE;
+}
+
+static void
+gnm_pane_drag_dest_init (GnmPane *pane, SheetControlGUI *scg)
+{
+	GtkWidget *widget = GTK_WIDGET (pane->gcanvas);
+
+	gtk_drag_dest_set (widget, GTK_DEST_DEFAULT_ALL,
+			   drag_types_in, G_N_ELEMENTS (drag_types_in),
+			   GDK_ACTION_COPY | GDK_ACTION_MOVE);
+	gtk_drag_dest_add_uri_targets (widget);
+	gtk_drag_dest_add_image_targets (widget);
+	gtk_drag_dest_add_text_targets (widget);
+
+	g_signal_connect (widget, "drag-data-received",
+		G_CALLBACK (cb_pane_drag_data_received), pane);
+	g_signal_connect (widget, "drag-data-get",
+		G_CALLBACK (cb_gnm_pane_drag_data_get), scg);
+	g_signal_connect (widget, "drag-motion",
+		G_CALLBACK (cb_gnm_pane_drag_motion), pane);
+ 	g_signal_connect (widget, "drag-leave",
+		G_CALLBACK (cb_gnm_pane_drag_leave), pane);
+}
+
 void
 gnm_pane_init (GnmPane *pane, SheetControlGUI *scg,
 	       gboolean col_headers, gboolean row_headers, int index)
@@ -187,6 +318,12 @@ gnm_pane_init (GnmPane *pane, SheetControlGUI *scg,
 		"popup-menu",
 		G_CALLBACK (cb_pane_popup_menu), pane);
 
+	if (NULL != scg &&
+	    NULL != (sheet = sc_sheet (SHEET_CONTROL (scg))) &&
+	    fabs (1. - sheet->last_zoom_factor_used) > 1e-6)
+		foo_canvas_set_pixels_per_unit (FOO_CANVAS (pane->gcanvas),
+						sheet->last_zoom_factor_used);
+
 	item = foo_canvas_item_new (pane->gcanvas->grid_items,
 		item_grid_get_type (),
 		"SheetControlGUI", scg,
@@ -223,12 +360,13 @@ gnm_pane_init (GnmPane *pane, SheetControlGUI *scg,
 	/* create views for the sheet objects */
 	sheet = sc_sheet (SHEET_CONTROL (scg));
 	if (sheet != NULL) {
-		GList *ptr;
+		GSList *ptr;
 		for (ptr = sheet->sheet_objects; ptr != NULL ; ptr = ptr->next)
 			sheet_object_new_view (ptr->data,
 				(SheetObjectViewContainer *)pane);
 	}
 
+	gnm_pane_drag_dest_init (pane, scg);
 	pane->mouse_cursor = NULL;
 }
 
@@ -273,9 +411,9 @@ gnm_pane_release (GnmPane *pane)
 	pane->grid = NULL;
 	pane->editor = NULL;
 	pane->cursor.std = pane->cursor.rangesel = pane->cursor.special = pane->cursor.rangehighlight = NULL;
-	pane->colrow_resize.guide = NULL;
-	pane->colrow_resize.start = NULL;
-	pane->colrow_resize.points = NULL;
+	pane->size_guide.guide = NULL;
+	pane->size_guide.start = NULL;
+	pane->size_guide.points = NULL;
 }
 
 void
@@ -297,29 +435,29 @@ gnm_pane_bound_set (GnmPane *pane,
 /****************************************************************************/
 
 void
-gnm_pane_colrow_resize_start (GnmPane *pane,
-			      gboolean is_cols, int resize_pos)
+gnm_pane_size_guide_start (GnmPane *pane, gboolean vert, int colrow, int width)
 {
 	SheetControlGUI const *scg;
 	GnmCanvas const *gcanvas;
 	FooCanvasPoints *points;
-	FooCanvasItem *item;
 	double zoom;
+	gboolean text_is_rtl;
 
 	g_return_if_fail (pane != NULL);
-	g_return_if_fail (pane->colrow_resize.guide  == NULL);
-	g_return_if_fail (pane->colrow_resize.start  == NULL);
-	g_return_if_fail (pane->colrow_resize.points == NULL);
+	g_return_if_fail (pane->size_guide.guide  == NULL);
+	g_return_if_fail (pane->size_guide.start  == NULL);
+	g_return_if_fail (pane->size_guide.points == NULL);
 
 	gcanvas = pane->gcanvas;
 	scg = gcanvas->simple.scg;
+	text_is_rtl = scg->sheet_control.sheet->text_is_rtl;
 	zoom = FOO_CANVAS (gcanvas)->pixels_per_unit;
 
-	points = pane->colrow_resize.points = foo_canvas_points_new (2);
-	if (is_cols) {
+	points = pane->size_guide.points = foo_canvas_points_new (2);
+	if (vert) {
 		double x = scg_colrow_distance_get (scg, TRUE,
-					0, resize_pos) / zoom;
-		if (scg->rtl)
+					0, colrow) / zoom;
+		if (text_is_rtl)
 			x = -x;
 		points->coords [0] = x;
 		points->coords [1] = scg_colrow_distance_get (scg, FALSE,
@@ -329,7 +467,7 @@ gnm_pane_colrow_resize_start (GnmPane *pane,
 					0, gcanvas->last_visible.row+1) / zoom;
 	} else {
 		double const y = scg_colrow_distance_get (scg, FALSE,
-					0, resize_pos) / zoom;
+					0, colrow) / zoom;
 		points->coords [0] = scg_colrow_distance_get (scg, TRUE,
 					0, gcanvas->first.col) / zoom;
 		points->coords [1] = y;
@@ -337,60 +475,79 @@ gnm_pane_colrow_resize_start (GnmPane *pane,
 					0, gcanvas->last_visible.col+1) / zoom;
 		points->coords [3] = y;
 
-		if (scg->rtl) {
+		if (text_is_rtl) {
 			points->coords [0] *= -1.;
 			points->coords [2] *= -1.;
 		}
 	}
 
-	/* Position the stationary only.  Guide line is handled elsewhere. */
-	item = foo_canvas_item_new (gcanvas->action_items,
+	/* Guideline positioning is done in gnm_pane_size_guide_motion */
+	pane->size_guide.guide = foo_canvas_item_new (gcanvas->action_items,
 		FOO_TYPE_CANVAS_LINE,
-		"points", points,
-		"fill_color", "black",
-		"width_pixels", 1,
+		"fill-color", "black",
+		"width-pixels", width,
 		NULL);
-	pane->colrow_resize.start = GTK_OBJECT (item);
 
-	item = foo_canvas_item_new (gcanvas->action_items,
-		FOO_TYPE_CANVAS_LINE,
-		"fill_color", "black",
-		"width_pixels", 1,
-		NULL);
-	pane->colrow_resize.guide = GTK_OBJECT (item);
+	/* cheat for now and differentiate between col/row resize and frozen panes
+	 * using the width.  Frozen pane guides do not require a start line */
+	if (width == 1)
+		pane->size_guide.start = foo_canvas_item_new (gcanvas->action_items,
+			FOO_TYPE_CANVAS_LINE,
+			"points", points,
+			"fill-color", "black",
+			"width-pixels", 1,
+			NULL);
+	else {
+		static char const dat [] = { 0x22, 0x88, 0x22, 0x88, 0x22, 0x88, 0x22, 0x88 };
+		GdkBitmap *stipple = gdk_bitmap_create_from_data (
+			GTK_WIDGET (pane->gcanvas)->window, dat, 8, 8);
+		foo_canvas_item_set (pane->size_guide.guide, "fill-stipple", stipple, NULL);
+		g_object_unref (stipple);
+	}
 }
 
 void
-gnm_pane_colrow_resize_stop (GnmPane *pane)
+gnm_pane_size_guide_stop (GnmPane *pane)
 {
 	g_return_if_fail (pane != NULL);
 
-	if (pane->colrow_resize.points != NULL) {
-		foo_canvas_points_free (pane->colrow_resize.points);
-		pane->colrow_resize.points = NULL;
+	if (pane->size_guide.points != NULL) {
+		foo_canvas_points_free (pane->size_guide.points);
+		pane->size_guide.points = NULL;
 	}
-	if (pane->colrow_resize.start != NULL) {
-		gtk_object_destroy (pane->colrow_resize.start);
-		pane->colrow_resize.start = NULL;
+	if (pane->size_guide.start != NULL) {
+		gtk_object_destroy (GTK_OBJECT (pane->size_guide.start));
+		pane->size_guide.start = NULL;
 	}
-	if (pane->colrow_resize.guide != NULL) {
-		gtk_object_destroy (pane->colrow_resize.guide);
-		pane->colrow_resize.guide = NULL;
+	if (pane->size_guide.guide != NULL) {
+		gtk_object_destroy (GTK_OBJECT (pane->size_guide.guide));
+		pane->size_guide.guide = NULL;
 	}
 }
 
+/**
+ * gnm_pane_size_guide_motion
+ * @pane : #GnmPane
+ * @vert : TRUE for a vertical guide, FALSE for horizontal
+ * @guide_pos : in unscaled sheet pixel coords
+ *
+ * Moves the guide line to @guide_pos.
+ * NOTE : gnm_pane_size_guide_start must be called before any calls to
+ *	gnm_pane_size_guide_motion
+ **/
 void
-gnm_pane_colrow_resize_move (GnmPane *pane, gboolean is_cols, int resize_pos)
+gnm_pane_size_guide_motion (GnmPane *pane, gboolean vert, int guide_pos)
 {
-	FooCanvasItem *resize_guide = FOO_CANVAS_ITEM (pane->colrow_resize.guide);
-	FooCanvasPoints *points     = pane->colrow_resize.points;
+	FooCanvasItem *resize_guide = FOO_CANVAS_ITEM (pane->size_guide.guide);
+	FooCanvasPoints *points     = pane->size_guide.points;
 	double const	 scale	    = 1. / resize_guide->canvas->pixels_per_unit;
 
-	if (is_cols)
+	if (vert) {
+		gboolean text_is_rtl = pane->gcanvas->simple.scg->sheet_control.sheet->text_is_rtl;
 		points->coords [0] = points->coords [2] = scale *
-			(pane->gcanvas->simple.scg->rtl ? -resize_pos : resize_pos);
-	else
-		points->coords [1] = points->coords [3] = scale * resize_pos;
+			(text_is_rtl ? -guide_pos : guide_pos);
+	} else
+		points->coords [1] = points->coords [3] = scale * guide_pos;
 
 	foo_canvas_item_set (resize_guide, "points",  points, NULL);
 }
@@ -568,6 +725,23 @@ gnm_pane_edit_stop (GnmPane *pane)
 	}
 }
 
+void
+gnm_pane_objects_drag (GnmPane *pane, SheetObject *so,
+		       gdouble new_x, gdouble new_y, int drag_type,
+		       gboolean symmetric,gboolean snap_to_grid)
+{
+ 	double dx, dy;
+ 	dx = new_x - pane->drag.last_x;
+ 	dy = new_y - pane->drag.last_y;
+	pane->drag.had_motion = TRUE;
+
+ 	scg_objects_drag (pane->gcanvas->simple.scg, pane->gcanvas,
+ 		so, &dx, &dy, drag_type, symmetric, snap_to_grid, TRUE);
+
+ 	pane->drag.last_x += dx;
+ 	pane->drag.last_y += dy;
+}
+
 #define CTRL_PT_SIZE		4
 #define CTRL_PT_OUTLINE		0
 /* space for 2 halves and a full */
@@ -582,18 +756,10 @@ gnm_pane_object_move (GnmPane *pane, GObject *ctrl_pt,
 {
 	int const idx = GPOINTER_TO_INT (g_object_get_data (ctrl_pt, "index"));
 	SheetObject *so  = g_object_get_data (G_OBJECT (ctrl_pt), "so");
- 	double dx, dy;
- 	dx = new_x - pane->drag.last_x;
- 	dy = new_y - pane->drag.last_y;
-	pane->drag.had_motion = TRUE;
- 	
- 	scg_objects_drag (pane->gcanvas->simple.scg, pane->gcanvas,
- 		so, &dx, &dy, idx, symmetric, snap_to_grid, TRUE);
- 
- 	pane->drag.last_x += dx;
- 	pane->drag.last_y += dy;
- 
-	if (idx != 8)
+
+	gnm_pane_objects_drag (pane, so, new_x, new_y, idx,
+			       symmetric, snap_to_grid);
+ 	if (idx != 8)
 		gnm_pane_display_obj_size_tip (pane, so);
 }
 
@@ -609,7 +775,7 @@ cb_slide_handler (GnmCanvas *gcanvas, int col, int row, gpointer ctrl_pt)
 	y = scg_colrow_distance_get (scg, FALSE, gcanvas->first.row, row);
 	y += gcanvas->first_offset.row;
 
-	if (scg->rtl)
+	if (scg->sheet_control.sheet->text_is_rtl)
 		x *= -1;
 
 	gnm_pane_object_move (gcanvas->pane, ctrl_pt, x * scale, y * scale, FALSE, FALSE);
@@ -733,6 +899,87 @@ control_point_set_cursor (SheetControlGUI const *scg, FooCanvasItem *ctrl_pt)
 	gnm_widget_set_cursor_type (GTK_WIDGET (ctrl_pt->canvas), cursor);
 }
 
+static void
+target_list_add_list (GtkTargetList *targets, GtkTargetList *added_targets)
+{
+	GList *ptr;
+	GtkTargetPair *tp;
+
+	g_return_if_fail (targets != NULL);
+
+	if (added_targets == NULL)
+		return;
+
+	for (ptr = added_targets->list; ptr !=  NULL; ptr = ptr->next) {
+		tp = (GtkTargetPair *)ptr->data;
+		gtk_target_list_add (targets, tp->target, tp->flags, tp->info);
+	}
+}
+
+/**
+ * Drag one or more sheet objects using GTK drag and drop, to the same
+ * sheet, another workbook, another gnumeric or a different application.
+ */
+static void
+gnm_pane_drag_begin (GnmPane *pane, SheetObject *so, GdkEvent *event)
+{
+	GdkDragContext *context;
+	GtkTargetList *targets, *im_targets;
+	FooCanvas *canvas    = FOO_CANVAS (pane->gcanvas);
+	SheetControlGUI *scg = pane->gcanvas->simple.scg;
+	GSList *objects;
+	SheetObject *imageable = NULL, *exportable = NULL;
+	GSList *ptr;
+	SheetObject *candidate;
+
+	targets = gtk_target_list_new (drag_types_out,
+				  G_N_ELEMENTS (drag_types_out));
+	objects = go_hash_keys (scg->selected_objects);
+	for (ptr = objects; ptr != NULL; ptr = ptr->next) {
+		candidate = SHEET_OBJECT (ptr->data);
+
+		if (exportable == NULL &&
+		    IS_SHEET_OBJECT_EXPORTABLE (candidate))
+			exportable = candidate;
+		if (imageable == NULL &&
+		    IS_SHEET_OBJECT_IMAGEABLE (candidate))
+			imageable = candidate;
+	}
+
+	if (exportable) {
+		im_targets = sheet_object_exportable_get_target_list (exportable);
+		if (im_targets != NULL) {
+			target_list_add_list (targets, im_targets);
+			gtk_target_list_unref (im_targets);
+		}
+	} 
+	if (imageable) {
+		im_targets = sheet_object_get_target_list (imageable);
+		if (im_targets != NULL) {
+			target_list_add_list (targets, im_targets);
+			gtk_target_list_unref (im_targets);
+		}
+	}
+#ifdef DEBUG_DND
+	{
+		GList *l;
+		g_print ("%d offered formats:\n", g_list_length (targets->list));
+		for (l = targets->list; l; l = l->next) {
+			GtkTargetPair *pair = (GtkTargetPair *)l->data;
+			char *target_name = gdk_atom_name (pair->target);
+			g_print ("%s\n", target_name);
+			g_free (target_name);
+ 		}
+	}
+#endif
+
+	context = gtk_drag_begin (GTK_WIDGET (canvas), targets,
+				  GDK_ACTION_COPY | GDK_ACTION_MOVE,
+				  pane->drag.button, event);
+	gtk_target_list_unref (targets);
+	g_slist_free (objects);
+}
+
 void
 gnm_pane_object_start_resize (GnmPane *pane, GdkEventButton *event,
 			      SheetObject *so, int drag_type, gboolean is_creation)
@@ -748,7 +995,7 @@ gnm_pane_object_start_resize (GnmPane *pane, GdkEventButton *event,
 
 	if (make_dup) {
 		SheetControlGUI *scg = pane->gcanvas->simple.scg;
-		GSList *ptr, *objs = gnm_hash_keys (scg->selected_objects);
+		GSList *ptr, *objs = go_hash_keys (scg->selected_objects);
 
 		for (ptr = objs ; ptr != NULL ; ptr = ptr->next) {
 			SheetObject *dup_obj = sheet_object_dup (ptr->data);
@@ -801,7 +1048,7 @@ cb_control_point_event (FooCanvasItem *ctrl_pt, GdkEvent *event, GnmPane *pane)
 
 		if (idx != 8) {
 			foo_canvas_item_set (ctrl_pt,
-				"fill_color",    "green",
+				"fill-color",    "green",
 				NULL);
 			gnm_pane_display_obj_size_tip (pane, so);
 		}
@@ -811,7 +1058,7 @@ cb_control_point_event (FooCanvasItem *ctrl_pt, GdkEvent *event, GnmPane *pane)
 		scg_set_display_cursor (scg);
 		if (idx != 8) {
 			foo_canvas_item_set (ctrl_pt,
-				"fill_color",    "white",
+				"fill-color",    "white",
 				NULL);
 			gnm_pane_clear_obj_size_tip (pane);
 		}
@@ -825,7 +1072,7 @@ cb_control_point_event (FooCanvasItem *ctrl_pt, GdkEvent *event, GnmPane *pane)
 
 	case GDK_BUTTON_RELEASE:
 		if (pane->drag.button != (int)event->button.button)
-			return FALSE;
+			break;
 		pane->drag.button = 0;
 		gnm_simple_canvas_ungrab (ctrl_pt, event->button.time);
 		gnm_canvas_slide_stop (gcanvas);
@@ -844,7 +1091,7 @@ cb_control_point_event (FooCanvasItem *ctrl_pt, GdkEvent *event, GnmPane *pane)
 
 	case GDK_BUTTON_PRESS:
 		if (0 != pane->drag.button)
-			return FALSE;
+			break;
 		switch (event->button.button) {
 		case 1:
 		case 2: gnm_pane_object_start_resize (pane, &event->button, so,  idx, FALSE);
@@ -870,10 +1117,12 @@ cb_control_point_event (FooCanvasItem *ctrl_pt, GdkEvent *event, GnmPane *pane)
 		 * the semantics of the col,row args in the callback.  However,
 		 * that is more work than I want to do right now.
 		 */
-		if (gnm_canvas_handle_motion (GNM_CANVAS (ctrl_pt->canvas),
-					      ctrl_pt->canvas, &event->motion,
-					      GNM_CANVAS_SLIDE_X | GNM_CANVAS_SLIDE_Y | GNM_CANVAS_SLIDE_EXTERIOR_ONLY,
-					      cb_slide_handler, ctrl_pt))
+		if (idx == 8 &&  !pane->drag.created_objects)
+			gnm_pane_drag_begin (pane, so, event);
+		else if (gnm_canvas_handle_motion (GNM_CANVAS (ctrl_pt->canvas),
+						   ctrl_pt->canvas, &event->motion,
+						   GNM_CANVAS_SLIDE_X | GNM_CANVAS_SLIDE_Y | GNM_CANVAS_SLIDE_EXTERIOR_ONLY,
+						   cb_slide_handler, ctrl_pt))
 			gnm_pane_object_move (pane, G_OBJECT (ctrl_pt),
 				event->motion.x, event->motion.y,
 				(event->button.state & GDK_CONTROL_MASK) != 0,
@@ -916,9 +1165,9 @@ new_control_point (GnmPane *pane, SheetObject *so, int idx, double x, double y)
 	item = foo_canvas_item_new (
 		gcanvas->action_items,
 		FOO_TYPE_CANVAS_ELLIPSE,
-		"outline_color", "black",
-		"fill_color",    "white",
-		"width_pixels",  CTRL_PT_OUTLINE,
+		"outline-color", "black",
+		"fill-color",    "white",
+		"width-pixels",  CTRL_PT_OUTLINE,
 		NULL);
 
 	g_signal_connect (G_OBJECT (item),
@@ -968,10 +1217,10 @@ set_acetate_coords (GnmPane *pane, SheetObject *so, FooCanvasItem **ctrl_pts,
 				GTK_WIDGET (pane->gcanvas)->window, dashed, 8, 8);
 			ctrl_pts [9] = foo_canvas_item_new (pane->gcanvas->action_items,
 				FOO_TYPE_CANVAS_RECT,
-				"fill_color",		NULL,
-				"width_units",   	1.,
-				"outline_color",	"black",
-				"outline_stipple",	stipple,
+				"fill-color",		NULL,
+				"width-units",   	1.,
+				"outline-color",	"black",
+				"outline-stipple",	stipple,
 				NULL);
 			g_object_unref (stipple);
 			foo_canvas_item_lower_to_bottom (ctrl_pts [9]);
@@ -1008,17 +1257,17 @@ set_acetate_coords (GnmPane *pane, SheetObject *so, FooCanvasItem **ctrl_pts,
 		FooCanvasItem *item = foo_canvas_item_new (
 			pane->gcanvas->action_items,
 			item_acetate_get_type (),
-			"fill_color",		NULL,
+			"fill-color",		NULL,
 #ifdef WITH_STIPPLE_BORDER
-			"width_units",		(double)(CTRL_PT_SIZE + CTRL_PT_OUTLINE),
-			"outline_color",	"black",
-			"outline_stipple",	stipple,
+			"width-units",		(double)(CTRL_PT_SIZE + CTRL_PT_OUTLINE),
+			"outline-color",	"black",
+			"outline-stipple",	stipple,
 #endif
 #if 0
 			/* work around the screwup in canvas-item-shape that adds a large
 			 * border to anything that uses miter (required for gnome-canvas
 			 * not foocanvas */
-			"join_style",		GDK_JOIN_ROUND,
+			"join-style",		GDK_JOIN_ROUND,
 #endif
 			NULL);
 #ifdef WITH_STIPPLE_BORDER
