@@ -1,9 +1,9 @@
-/* vim: set sw=8: */
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 
 /*
  * sheet-view.c:
  *
- * Copyright (C) 2002 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2002-2005 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of version 2 of the GNU General Public License as published
@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 #include <gnumeric-config.h>
@@ -59,8 +59,8 @@ cb_update_auto_expr (gpointer data)
 {
 	SheetView *sv = (SheetView *) data;
 
-	if (wb_view_cur_sheet_view (sv->wbv) == sv)
-		wb_view_auto_expr_recalc (sv->wbv, TRUE);
+	if (wb_view_cur_sheet_view (sv->sv_wbv) == sv)
+		wb_view_auto_expr_recalc (sv->sv_wbv, TRUE);
 
 	sv->auto_expr_timer = 0;
 	return FALSE;
@@ -68,6 +68,14 @@ cb_update_auto_expr (gpointer data)
 
 /*************************************************************************/
 
+static void
+sv_sheet_name_changed (G_GNUC_UNUSED Sheet *sheet,
+		       G_GNUC_UNUSED GParamSpec *pspec,
+		       SheetView *sv)
+{
+	sv->edit_pos_changed.content = TRUE;
+}
+
 Sheet *
 sv_sheet (SheetView const *sv)
 {
@@ -77,7 +85,7 @@ sv_sheet (SheetView const *sv)
 WorkbookView *
 sv_wbv (SheetView const *sv)
 {
-	return sv->wbv;
+	return sv->sv_wbv;
 }
 
 static void
@@ -85,7 +93,7 @@ sv_init_sc (SheetView const *sv, SheetControl *sc)
 {
 	GnmCellPos initial;
 
-	sc_set_zoom_factor (sc);
+	sc_scale_changed (sc);
 
 	/* set_panes will change the initial so cache it */
 	initial = sv->initial_top_left;
@@ -115,6 +123,13 @@ sv_attach_control (SheetView *sv, SheetControl *sc)
 	sv_init_sc (sv, sc);
 }
 
+SheetControl *
+sv_get_control (SheetView const *sv, WorkbookControl const *wbc)
+{
+	SHEET_VIEW_FOREACH_CONTROL (sv, sc, if (sc_wbc (sc) == wbc) return sc;);
+	return NULL;
+}
+
 void
 sv_detach_control (SheetControl *sc)
 {
@@ -163,8 +178,9 @@ sv_weak_unref (SheetView **ptr)
 }
 
 static GObjectClass *parent_class;
+
 static void
-s_view_finalize (GObject *object)
+sv_real_dispose (GObject *object)
 {
 	SheetView *sv = SHEET_VIEW (object);
 
@@ -177,11 +193,20 @@ s_view_finalize (GObject *object)
 			g_warning ("Unexpected left over controls");
 	}
 
+	if (sv->sheet) {
+		Sheet *sheet = sv->sheet;
+		sv->sheet = NULL;
+		g_ptr_array_remove (sheet->sheet_views, sv);
+		g_signal_handlers_disconnect_by_func (sheet, sv_sheet_name_changed, sv);
+		g_object_unref (sv);
+		g_object_unref (sheet);
+	}
+
 	sv_unant (sv);
 	sv_selection_free (sv);
 	auto_expr_timer_clear (sv);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	parent_class->dispose (object);
 }
 
 static void
@@ -192,7 +217,7 @@ sheet_view_class_init (GObjectClass *klass)
 	g_return_if_fail (wbc_class != NULL);
 
 	parent_class = g_type_class_peek_parent (klass);
-	klass->finalize = s_view_finalize;
+	klass->dispose = sv_real_dispose;
 }
 
 static void
@@ -226,15 +251,32 @@ GSF_CLASS (SheetView, sheet_view,
 SheetView *
 sheet_view_new (Sheet *sheet, WorkbookView *wbv)
 {
-	SheetView *sv = g_object_new (SHEET_VIEW_TYPE, NULL);
-	sheet_attach_view (sheet, sv);
-	sv->wbv = wbv;
+	SheetView *sv;
+
+	g_return_val_if_fail (IS_SHEET (sheet), NULL);
+
+	sv = g_object_new (SHEET_VIEW_TYPE, NULL);
+	sv->sheet = g_object_ref (sheet);
+	sv->sv_wbv = wbv;
+	g_ptr_array_add (sheet->sheet_views, sv);
+	g_object_ref (sv);
+
+	g_signal_connect (G_OBJECT (sheet),
+			  "notify::name",
+			  G_CALLBACK (sv_sheet_name_changed),
+			  sv);
 
 	SHEET_VIEW_FOREACH_CONTROL (sv, control,
 		sv_init_sc (sv, control););
 	return sv;
 }
 
+void
+sv_dispose (SheetView *sv)
+{
+	g_object_run_dispose (G_OBJECT (sv));
+}
+
 void
 sv_unant (SheetView *sv)
 {
@@ -310,7 +352,7 @@ sv_selection_copy (SheetView *sv, WorkbookControl *wbc)
 {
 	GnmRange const *sel;
 
-	if (!(sel = selection_first_range (sv, GNM_CMD_CONTEXT (wbc), _("Copy"))))
+	if (!(sel = selection_first_range (sv, GO_CMD_CONTEXT (wbc), _("Copy"))))
 		return FALSE;
 
 	gnm_app_clipboard_cut_copy (wbc, FALSE, sv, sel, TRUE);
@@ -335,10 +377,10 @@ sv_selection_cut (SheetView *sv, WorkbookControl *wbc)
 	 */
 	g_return_val_if_fail (IS_SHEET_VIEW (sv), FALSE);
 
-	if (!(sel = selection_first_range (sv, GNM_CMD_CONTEXT (wbc), _("Cut"))))
+	if (!(sel = selection_first_range (sv, GO_CMD_CONTEXT (wbc), _("Cut"))))
 		return FALSE;
 
-	if (sheet_range_splits_region (sv_sheet (sv), sel, NULL, GNM_CMD_CONTEXT (wbc), _("Cut")))
+	if (sheet_range_splits_region (sv_sheet (sv), sel, NULL, GO_CMD_CONTEXT (wbc), _("Cut")))
 		return FALSE;
 
 	gnm_app_clipboard_cut_copy (wbc, TRUE, sv, sel, TRUE);
@@ -540,19 +582,19 @@ sv_update (SheetView *sv)
 {
 	if (sv->edit_pos_changed.content) {
 		sv->edit_pos_changed.content = FALSE;
-		if (wb_view_cur_sheet_view (sv->wbv) == sv)
-			wb_view_edit_line_set (sv->wbv, NULL);
+		if (wb_view_cur_sheet_view (sv->sv_wbv) == sv)
+			wb_view_edit_line_set (sv->sv_wbv, NULL);
 	}
 
 	if (sv->edit_pos_changed.format) {
 		sv->edit_pos_changed.format = FALSE;
-		if (wb_view_cur_sheet_view (sv->wbv) == sv)
-			wb_view_format_feedback (sv->wbv, TRUE);
+		if (wb_view_cur_sheet_view (sv->sv_wbv) == sv)
+			wb_view_format_feedback (sv->sv_wbv, TRUE);
 	}
 
 	if (sv->edit_pos_changed.location) {
 		sv->edit_pos_changed.location = FALSE;
-		if (wb_view_cur_sheet_view (sv->wbv) == sv) {
+		if (wb_view_cur_sheet_view (sv->sv_wbv) == sv) {
 			char const *new_pos = sheet_names_check (sv->sheet,
 				selection_first_range (sv, NULL, NULL));
 			if (new_pos == NULL)
@@ -679,7 +721,7 @@ sv_freeze_panes (SheetView *sv,
 	SHEET_VIEW_FOREACH_CONTROL (sv, control,
 		sv_init_sc (sv, control););
 
-	WORKBOOK_VIEW_FOREACH_CONTROL(sv->wbv, wbc,
+	WORKBOOK_VIEW_FOREACH_CONTROL(sv->sv_wbv, wbc,
 		wb_control_menu_state_update (wbc, MS_FREEZE_VS_THAW););
 }
 
