@@ -38,9 +38,8 @@
 #include "workbook.h"
 #include "workbook-control.h"
 #include "command-context.h"
-#include "format.h"
+#include "gnm-format.h"
 #include "sheet-object-cell-comment.h"
-#include "commands.h"
 #include "style-color.h"
 
 #include <glib/gi18n.h>
@@ -117,7 +116,7 @@ dao_range_name (data_analysis_output_t *dao)
 		    dao->start_col + dao->cols - 1, 
 		    dao->start_row + dao->rows - 1);
 
-	return cmd_range_to_str_utility (dao->sheet, &range);
+	return undo_range_name (dao->sheet, &range);
 }
 
 /**
@@ -126,7 +125,7 @@ dao_range_name (data_analysis_output_t *dao)
  * @format:
  * @result:
  *
- * Uses fmt to provide a string to be used as command descriptor for
+ * Uses format to provide a string to be used as command descriptor for
  * undo/redo
  *
  **/
@@ -224,14 +223,14 @@ dao_prepare_output (WorkbookControl *wbc, data_analysis_output_t *dao,
 		dao->start_col = dao->start_row = 0;
 		dao->rows = SHEET_MAX_ROWS;
 		dao->cols = SHEET_MAX_COLS;
-		workbook_sheet_attach (wb, dao->sheet, NULL);
+		workbook_sheet_attach (wb, dao->sheet);
 	} else if (dao->type == NewWorkbookOutput) {
 		Workbook *wb = workbook_new ();
 		dao->sheet = sheet_new (wb, name);
 		dao->start_col = dao->start_row = 0;
 		dao->rows = SHEET_MAX_ROWS;
 		dao->cols = SHEET_MAX_COLS;
-		workbook_sheet_attach (wb, dao->sheet, NULL);
+		workbook_sheet_attach (wb, dao->sheet);
 		dao->wbc = wb_control_wrapper_new (dao->wbc, NULL, wb, NULL);
 	}
 	if (dao->rows == 0 || (dao->rows == 1 && dao->cols == 1))
@@ -263,7 +262,7 @@ dao_format_output (data_analysis_output_t *dao, char const *cmd)
 
 	if (dao->type == RangeOutput
 	    && sheet_range_splits_region (dao->sheet, &range, NULL,
-					  GNM_CMD_CONTEXT (dao->wbc), cmd))
+					  GO_CMD_CONTEXT (dao->wbc), cmd))
 		return TRUE;
 
 	if (dao->clear_outputrange)
@@ -277,7 +276,7 @@ dao_format_output (data_analysis_output_t *dao, char const *cmd)
 			    range.start.col, range.start.row,
 			    range.end.col, range.end.row,
 			    clear_flags | CLEAR_NOCHECKARRAY | CLEAR_MERGES,
-			    GNM_CMD_CONTEXT (dao->wbc));
+			    GO_CMD_CONTEXT (dao->wbc));
 	return FALSE;
 }
 
@@ -634,11 +633,11 @@ dao_set_style (data_analysis_output_t *dao, int col1, int row1,
 		range.end.row = dao->start_row + dao->rows;
 
 	if (range.end.col < range.start.col) {
-		mstyle_unref (mstyle);
+		gnm_style_unref (mstyle);
 		return;
 	}
 	if (range.end.row < range.start.row) {
-		mstyle_unref (mstyle);
+		gnm_style_unref (mstyle);
 		return;
 	}
 
@@ -661,7 +660,7 @@ void
 dao_set_bold (data_analysis_output_t *dao, int col1, int row1,
 	      int col2, int row2)
 {
-	GnmStyle *mstyle = mstyle_new ();
+	GnmStyle *mstyle = gnm_style_new ();
 	GnmRange  range;
 
 	range.start.col = col1 + dao->start_col;
@@ -669,7 +668,7 @@ dao_set_bold (data_analysis_output_t *dao, int col1, int row1,
 	range.end.col   = col2 + dao->start_col;
 	range.end.row   = row2 + dao->start_row;
 
-	mstyle_set_font_bold (mstyle, TRUE);
+	gnm_style_set_font_bold (mstyle, TRUE);
 	sheet_style_apply_range (dao->sheet, &range, mstyle);
 }
 
@@ -689,7 +688,7 @@ void
 dao_set_underlined (data_analysis_output_t *dao, int col1, int row1,
 		    int col2, int row2)
 {
-	GnmStyle *mstyle = mstyle_new ();
+	GnmStyle *mstyle = gnm_style_new ();
 	GnmRange  range;
 
 	range.start.col = col1 + dao->start_col;
@@ -697,7 +696,7 @@ dao_set_underlined (data_analysis_output_t *dao, int col1, int row1,
 	range.end.col   = col2 + dao->start_col;
 	range.end.row   = row2 + dao->start_row;
 
-	mstyle_set_font_uline (mstyle, TRUE);
+	gnm_style_set_font_uline (mstyle, TRUE);
 	sheet_style_apply_range (dao->sheet, &range, mstyle);
 }
 
@@ -717,9 +716,9 @@ void
 dao_set_italic (data_analysis_output_t *dao, int col1, int row1,
 		int col2, int row2)
 {
-	GnmStyle *mstyle = mstyle_new ();
+	GnmStyle *mstyle = gnm_style_new ();
 
-	mstyle_set_font_italic (mstyle, TRUE);
+	gnm_style_set_font_italic (mstyle, TRUE);
 	dao_set_style (dao, col1, row1, col2, row2, mstyle);
 }
 
@@ -739,12 +738,8 @@ void
 dao_set_percent (data_analysis_output_t *dao, int col1, int row1,
 		 int col2, int row2)
 {
-	GnmStyle *mstyle = mstyle_new ();
-	GnmFormat *style_format = NULL;
-
-	style_format = style_format_default_percentage ();
-	mstyle_set_format (mstyle, style_format);
-
+	GnmStyle *mstyle = gnm_style_new ();
+	gnm_style_set_format (mstyle, go_format_default_percentage ());
 	dao_set_style (dao, col1, row1, col2, row2, mstyle);
 }
 
@@ -764,14 +759,9 @@ void
 dao_set_date (data_analysis_output_t *dao, int col1, int row1,
 		 int col2, int row2)
 {
-	GnmStyle *mstyle = mstyle_new ();
-	GnmFormat *style_format = NULL;
-
-	style_format = style_format_default_date ();
-	mstyle_set_format (mstyle, style_format);
-
-	dao_set_style (dao, col1, row1,
-		       col2, row2, mstyle);
+	GnmStyle *mstyle = gnm_style_new ();
+	gnm_style_set_format (mstyle, go_format_default_date ());
+	dao_set_style (dao, col1, row1, col2, row2, mstyle);
 }
 
 /**
@@ -793,10 +783,10 @@ dao_set_colors (data_analysis_output_t *dao, int col1, int row1,
 {
 	GnmStyle *mstyle;
 
-	mstyle = mstyle_new ();
-	mstyle_set_color (mstyle, MSTYLE_COLOR_FORE, fore);
-	mstyle_set_color (mstyle, MSTYLE_COLOR_BACK, back);
-	mstyle_set_pattern (mstyle, 1);
+	mstyle = gnm_style_new ();
+	gnm_style_set_font_color (mstyle, fore);
+	gnm_style_set_back_color (mstyle, back);
+	gnm_style_set_pattern (mstyle, 1);
 	dao_set_style (dao, col1, row1,
 		       col2, row2, mstyle);
 }
@@ -816,13 +806,13 @@ dao_set_colors (data_analysis_output_t *dao, int col1, int row1,
 void
 dao_set_align (data_analysis_output_t *dao, int col1, int row1,
 	       int col2, int row2,
-	       StyleHAlignFlags align_h, StyleVAlignFlags align_v)
+	       GnmHAlign align_h, GnmVAlign align_v)
 {
 	GnmStyle *mstyle;
 
-	mstyle = mstyle_new ();
-	mstyle_set_align_h (mstyle, align_h);
-	mstyle_set_align_v (mstyle, align_v);
+	mstyle = gnm_style_new ();
+	gnm_style_set_align_h (mstyle, align_h);
+	gnm_style_set_align_v (mstyle, align_v);
 	dao_set_style (dao, col1, row1, col2, row2, mstyle);
 }
 
